@@ -4,21 +4,38 @@ except ImportError:
     ifilter = filter
 
 
+class SequenceIterator(object):
+    def __init__(self, items):
+        self.items = items
+        self.i = -1
+
+    def next(self):
+        try:
+            self.i += 1
+            return self.items[self.i]
+        except IndexError:
+            raise StopIteration
+
+
 def first(sequence, message=None):
     """The first item in that sequence
 
     If there aren't any, raise a ValueError with that message
+
+    >>> assert first([1, 2, 3]) == 1
     """
     try:
         return next(iter(sequence))
     except StopIteration:
-        raise ValueError(message or ('Sequence is empty: %s' % sequence))
+        raise ValueError(message or (f'Sequence is empty: {sequence}'))
 
 
 def last(sequence, message=None):
     """The last item in that sequence
 
     If there aren't any, raise a ValueError with that message
+
+    >>> assert last([1, 2, 3]) == 3
     """
     try:
         return sequence.pop()
@@ -29,6 +46,11 @@ def last(sequence, message=None):
 
 
 def first_or(sequence, value):
+    """First item in that sequence, or that value
+
+    >>> assert first_or([1, 2, 3], 4) == 1
+    >>> assert first_or([], 4) == 4
+    """
     try:
         return first(sequence)
     except ValueError:
@@ -39,6 +61,8 @@ def first_that(predicate, sequence, message=None):
     """The first item in that sequence that matches that predicate
 
     If none matches raise a KeyError with that message
+
+    >>> assert first_that(lambda x: x > 1, [1, 2, 3]) == 2
     """
     try:
         return next(ifilter(predicate, sequence))
