@@ -6,7 +6,7 @@ class NoTerminalAvailable(NotImplementedError):
 
 
 def _tput(tput_command):
-    command = 'tput %s' % tput_command
+    command = f'tput {tput_command}'
     status, output = cmnds.getstatusoutput(command)
     if status:
         if 'No value for $TERM' in output:
